@@ -31,6 +31,7 @@ Add the files or directories passed as arguments to the
 repository. When a directory name is given, all the files and
 subdirectories are recursively added."""
 
+directory = DirectoryHasRepository(needs_current_series = False)
 options = []
 
 
@@ -40,7 +41,4 @@ def func(parser, options, args):
     if len(args) < 1:
         parser.error('incorrect number of arguments')
 
-    if not crt_series.get_current():
-        raise CmdException, 'No patches applied'
-
     git.add(args)
