@@ -31,6 +31,7 @@ necessary pop and push operations will be performed to accomplish
 this. The '--series' option can be used to rearrange the (top) patches
 as specified by the given series file (or the standard input)."""
 
+directory = DirectoryGotoToplevel()
 options = [make_option('-s', '--series',
                        help = 'rearrange according to a series file',
                        action = 'store_true')]
@@ -45,7 +46,7 @@ def func(parser, options, args):
 
     check_local_changes()
     check_conflicts()
-    check_head_top_equal()
+    check_head_top_equal(crt_series)
 
     unapplied = crt_series.get_unapplied()
     applied = crt_series.get_applied()
@@ -81,10 +82,17 @@ def func(parser, options, args):
             topop.append(top)
     topush = patches + topush
 
+    # remove common patches to avoid unnecessary pop/push
+    while topush and topop:
+        if topush[-1] != topop[-1]:
+            break
+        topush.pop()
+        topop.pop()
+
     # check whether the operation is really needed
     if topop != topush:
         if topop:
-            pop_patches(topop)
+            pop_patches(crt_series, topop)
         if topush:
             topush.reverse()
-            push_patches(topush)
+            push_patches(crt_series, topush)
